@@ -18,13 +18,12 @@
 # Creation: 08 Nov 2011
 ###############################################################################
 
-library(pkgmaker)
-
 ###% Returns all the libraries that provides a user-supplied RNG
 ###% 
 ###% The library that provides the wrapper hooks for the management multiple 
 ###% user-supplied RNG is removed from the output list.
 ###% 
+#' @importFrom utils tail
 RNGlibs <- function(n=0, full=FALSE, hook="user_unif_rand", unlist=TRUE){
 	dlls <- getLoadedDLLs()
 	res <- lapply(dlls, function(d){
@@ -110,7 +109,12 @@ RNGprovider <- function(kind=RNGkind(), user.supplied=FALSE){
 
 #' Directly Getting or Setting the RNG Seed
 #' 
-#' \code{RNGseed} directly gets/sets the current RNG seed \code{.Random.seed}.
+#' These functions provide a direct access to the RNG seed object `.Random.seed`.
+#' 
+#' @name RNGseed
+NULL
+
+#' @describeIn RNGseed directly gets/sets the current RNG seed \code{.Random.seed}.
 #' It can typically be used to backup and restore the RNG state on exit of 
 #' functions, enabling local RNG changes.
 #' 
@@ -170,15 +174,15 @@ RNGseed <- function(seed){
 	invisible(res)
 }
 
-#' \code{RNGrecovery} recovers from a broken state of \code{.Random.seed}, 
+#' @describeIn RNGseed recovers from a broken state of \code{.Random.seed}, 
 #' and reset the RNG settings to defaults.
 #' 
 #' @export
-#' @rdname RNGseed
 RNGrecovery <- function(){
 	s <- as.integer(c(401,0,0))
 	assign(".Random.seed", s, envir=.GlobalEnv)
-	RNGkind("default", "default")
+	do.call(RNGkind, as.list(rep("default", .RNGkind_length())))
+  
 }
 
 .getRNGattribute <- function(object){
@@ -290,39 +294,46 @@ hasRNG <- function(object){
 	!is.null(.getRNGattribute(object))
 } 
 
+#' Getting RNG Seeds
+#' 
 #' \code{.getRNG} is an S4 generic that extract RNG settings from a variety of 
 #' object types.
 #' Its methods define the workhorse functions that are called by \code{getRNG}.
 #' 
-#' @rdname rng
-#' @inline
+#' @inheritParams getRNG
 #' @export
 setGeneric('.getRNG', function(object, ...) standardGeneric('.getRNG') )
-#' Default method that tries to extract RNG information from \code{object}, by 
+
+#' @describeIn .getRNG Default method that tries to extract RNG information from \code{object}, by 
 #' looking sequentially to a slot named \code{'rng'}, a slot named \code{'rng.seed'}
 #' or an attribute names \code{'rng'}.
 #' 
 #' It returns \code{NULL} if no RNG data was found.
 setMethod('.getRNG', 'ANY',
 	function(object, ...){
-		.getRNGattribute(object)
-	}
-)
-#' Returns the current RNG settings.
-setMethod('.getRNG', 'missing',
-	function(object){
-		
-		# return current value of .Random.seed
-		# ensuring it exists first 
-		if( !exists('.Random.seed', envir = .GlobalEnv) ) 
-			sample(NA)
-		
-		return( get('.Random.seed', envir = .GlobalEnv) )
-		
+	  if( missing(object) ) .get_Random.seed() # safe-guard
+		else .getRNGattribute(object)
 	}
 )
 
-#' Method for S3 objects, that aims at reproducing the behaviour of the function 
+.get_Random.seed <- function(){
+  # return current value of .Random.seed
+  # ensuring it exists first 
+  if( !exists('.Random.seed', envir = .GlobalEnv) ) 
+    sample(NA)
+  
+  return( get('.Random.seed', envir = .GlobalEnv) )
+  
+}
+#' @describeIn .getRNG Returns the current RNG settings.
+setMethod('.getRNG', 'missing',
+	function(object){
+	  .get_Random.seed()
+	  
+	}
+)
+
+#' @describeIn .getRNG Method for S3 objects, that aims at reproducing the behaviour of the function 
 #' \code{getRNG} of the package \code{getRNG}. 
 #' 
 #' It sequentially looks for RNG data in elements \code{'rng'}, \code{noise$rng} 
@@ -341,7 +352,7 @@ setMethod('.getRNG', 'list',
 #			object	
 #		}
 #)
-#' Method for numeric vectors, which returns the object itself, coerced into an integer 
+#' @describeIn .getRNG Method for numeric vectors, which returns the object itself, coerced into an integer 
 #' vector if necessary, as it is assumed to already represent a value for 
 #' \code{\link{.Random.seed}}.
 #' 
@@ -351,9 +362,11 @@ setMethod('.getRNG', 'numeric',
 	}
 )
 
+#' Extracting RNG Settings from Computation Result Objects
+#' 
 #' \code{getRNG1} is an S4 generic that returns the \strong{initial} RNG settings 
 #' used for computing an object.
-#' For example, in the case of results from multiple model fies, it would 
+#' For example, in the case of results from multiple model fits, it would 
 #' return the RNG settings used to compute the \emph{first} fit.
 #' 
 #' \code{getRNG1} is defined to provide separate access to the RNG settings as 
@@ -362,17 +375,17 @@ setMethod('.getRNG', 'numeric',
 #' result only.
 #' 
 #' Think of a sequence of separate computations, from which only one result is 
-#' used for the result (e.g. the one that maximises a likelihood): 
+#' used for the result (e.g. the one that maximizes a likelihood): 
 #' \code{getRNG1} would return the RNG settings to reproduce the complete sequence
 #' of computations, while \code{getRNG} would return the RNG settings necessary to 
 #' reproduce only the computation whose result has maximum likelihood.  
 #' 
-#' @rdname rng
-#' @export
-#' @inline
+#' @param object an R object.
+#' @param ... extra arguments to allow extension.
 #' 
+#' @export
 setGeneric('getRNG1', function(object, ...) standardGeneric('getRNG1') )
-#' Default method that is identical to \code{getRNG(object, ...)}.
+#' @describeIn getRNG1 Default method that is identical to \code{getRNG(object, ...)}.
 setMethod('getRNG1', 'ANY',
 	function(object, ...){
 		getRNG(object, ...)
@@ -428,6 +441,7 @@ nextRNG <- function(object, ..., ndraw=0L){
 	res
 }
 
+#' @importFrom utils head
 .collapse <- function(x, sep=', ', n=7L){
 	
 	res <- paste(head(x, n), collapse=', ')
@@ -439,6 +453,9 @@ nextRNG <- function(object, ..., ndraw=0L){
 #' \code{setRNG} set the current RNG with a seed, 
 #' using a suitable \code{.setRNG} method to set these settings.
 #'
+#' @param verbose a logical that indicates if the new RNG settings should
+#' be displayed.
+#' 
 #' @param check logical that indicates if only valid RNG kinds should be
 #' accepted, or if invalid values should just throw a warning.
 #' Note that this argument is used only on R >= 3.0.2.
@@ -474,20 +491,23 @@ setRNG <- function(object, ..., verbose=FALSE, check = TRUE){
 	})
 
 	# call S4 method on object
-    # check validity of the seed
-	tryCatch(.setRNG(object, ...)
+	# check validity of the seed
+	invalid_seed <- NULL
+	withCallingHandlers(.setRNG(object, ...)
             , warning = function(err){
-                if( check && testRversion('> 3.0.1') 
-                        && grepl("\\.Random\\.seed.* is not a valid", err$message) ){
+              invalid_seed <<- grepl("\\.Random\\.seed.* is not a valid", err$message)
+                if( check && testRversion('> 3.0.1') && invalid_seed ){
                     stop("setRNG - Invalid RNG kind [", str_out(object), "]: "
-							, err$message, '.'
-							, call.=FALSE)
-                }else{
-                    warning(err)
+                         , err$message, '.'
+                         , call.=FALSE)
+                }else if( !invalid_seed ){ # if the seed is not invalid then we show the warning and continue
+                  warning(err)
+                  invokeRestart("muffleWarning")
+                  
                 }
-            } 
+            }
     )
-	
+	if( isTRUE(invalid_seed) ) return()
 	# cancel RNG restoration
 	on.exit()
 	if( verbose ) showRNG()			
@@ -495,20 +515,19 @@ setRNG <- function(object, ..., verbose=FALSE, check = TRUE){
 	invisible(orseed)
 }
 
+#' Setting RNG Seeds
+#' 
 #' \code{.setRNG} is an S4 generic that sets the current RNG settings, from a 
 #' variety of specifications.
 #' Its methods define the workhorse functions that are called by \code{setRNG}.
 #' 
-#' @inline
-#' @rdname rng
+#' @inheritParams setRNG
 #' @export 
 setGeneric('.setRNG', function(object, ...) standardGeneric('.setRNG') )
-#' Sets the RNG to kind \code{object}, assuming is a valid RNG kind:
-#' it is equivalent to \code{RNGkind(object, ...}.
-#' All arguments in \code{...} are passed to \code{\link{RNGkind}}.
-#' 
-#' @param verbose a logical that indicates if the new RNG settings should
-#' be displayed.
+#' @describeIn .setRNG Sets the RNG to the kind(s) specified in \code{object}.
+#' If object is a single string that is a valid RNG kind, then this method is equivalent to \code{RNGkind(object, ...}.
+#' Otherwise, each element is assumed to be a valid argument for [RNGkind].
+#' Note that this latter case the names of `object`, if any, are used as argument names in the call to [RNGkind]. 
 #' 
 #' @examples
 #' # set RNG kind
@@ -519,12 +538,19 @@ setMethod('.setRNG', 'character',
 	function(object, ...){
 		if( length(object) == 1L )
 			RNGkind(kind=object, ...)
-		else
-			RNGkind(kind=object[1L], normal.kind=object[2L])
+		else{
+		  n0 <- .RNGkind_length()
+		  if( length(object) > n0 ){
+		    warning("RNG character specification is too long: discarding elements ", paste0(tail(object, -n0), collapse = ", "))
+		    
+		  }
+			do.call(RNGkind, as.list(head(object, n0)))
+		  
+		}
 	}
 )
 
-#' Sets the RNG settings using \code{object} directly the new value for 
+#' @describeIn .setRNG Sets the RNG settings using \code{object} directly the new value for 
 #' \code{.Random.seed} or to initialise it with \code{\link{set.seed}}.
 #' 
 #' @examples 
@@ -570,7 +596,20 @@ setMethod('.setRNG', 'numeric',
 					, .collapse(seed, n=5), "]: ", err$message, '.'
 					, call.=FALSE)
 			    }
-            )
+			, warning = function(w){
+			  stop("setRNG - Invalid numeric seed ["
+			       , .collapse(seed, n=5), "]: ", w$message, '.'
+			       , call.=FALSE)
+			}
+			, finally = {
+			  if( !identical(seed[1L], RNGseed()[1L]) ){
+			    msg <- "detected that the RNG kind would change after frist draw."
+			    stop("setRNG - Invalid numeric seed ["
+			         , .collapse(seed, n=5), "]: ", msg, '.'
+			         , call.=FALSE)
+			  }
+			})
+			# re-force setting the seed if no error happened
 			RNGseed(seed)			
 		}
 	}
